/**
 * 校验参数工具
 * 
 */

import { ERRORENUM } from "../config/errorEnum";
import { BizError } from "./bizError";

/**
 * 通过config校验表单参数param
 * 包括类型 String, Number, Boolean, [Number], [Object]
 * 参数是必填
 * 方法会校验表单中存在的多余字段
 * todo 后续优化配置
 * @param name 被调用的方法名
 * @param config 校验配置
 * @param param 需要校验的参数
 * @param skipKeys 跳过验证的字段
 * @returns true 无需关注返回
 */
export function eccFormParam(name:string, keyTypeConf:object, param:object, skipKeys?) {
    for (let key in keyTypeConf ) {
        if ( skipKeys && skipKeys.indexOf(key) > -1 && !param[key]) {
            continue;
        }
        if ( typeof param[key] != 'boolean' && !param[key] ) throw new BizError(ERRORENUM.表单校验失败, name, `缺失${key}字段`);
        let type = typeof param[key];
        switch(keyTypeConf[key]) {
            case 'Number':
                if ( type != 'number' ) throw new BizError(ERRORENUM.表单校验失败, name, `${key}应是number型 而不是${type}`);
            break;
            case 'String':
                if ( type != 'string' ) throw new BizError(ERRORENUM.表单校验失败, name, `${key}应是string型 而不是${type}`);
            break;
            case 'Boolean':
                if ( type != 'boolean' ) throw new BizError(ERRORENUM.表单校验失败, name, `${key}应是boolean型 而不是${type}`); 
            break;
            case '[Number]':
                if ( !Array.isArray(param[key]) ) throw new BizError(ERRORENUM.表单校验失败, name, `${key}应是数组型 而不是${type}`);
                for (let i =0; i <  param[key].length; i++) {
                    let item = param[key][i];
                    if ( typeof item != 'number' ) {
                        throw new BizError(ERRORENUM.表单校验失败, name, `${key}应是number型数组其中下标${i}是${typeof item}`);
                    }
                }
            break;
            case '[Object]':
                if ( !Array.isArray(param[key]) ) throw new BizError(ERRORENUM.表单校验失败, name, `${key}应是数组型 而不是${type}`);
                for (let i =0; i <  param[key].length; i++) {
                    let item = param[key][i];
                    if ( typeof item != 'object' ) {
                        throw new BizError(ERRORENUM.表单校验失败, name, `${key}应是object型数组其中下标${i}是${typeof item}`);
                    }
                }
            break;
        }
    }
    for (let key in param) {
        if (!keyTypeConf[key]) throw new BizError(ERRORENUM.表单校验失败, name, `多余${key}字段`);
    }
    return true;
}


/**
 * 根据conf配置校验请求参数
 * 参数为必填
 * @param conf 配置
 * @param param 待校验对象
 * @returns 原有表单数据
 */
export function eccReqMustHaveParam(conf:object, param) {
    for (let key in conf ) {
        if ( typeof param[key] != 'boolean' && !param[key] ) throw new BizError(ERRORENUM.参数错误, `缺失${key}字段`);
        let type = typeof param[key];
        switch(conf[key]) {
            case 'Number':
                if ( type != 'number' ) throw new BizError(ERRORENUM.参数错误, `${key}应是number型 而不是${type}`);
            break;
            case 'String':
                if ( type != 'string' ) throw new BizError(ERRORENUM.参数错误, `${key}应是string型 而不是${type}`);
            break;
            case 'Boolean':
                if ( type != 'boolean' ) throw new BizError(ERRORENUM.参数错误, `${key}应是boolean型 而不是${type}`); 
            break;
            case '[Number]':
                if ( !Array.isArray(param[key]) ) throw new BizError(ERRORENUM.参数错误, `${key}应是数组型 而不是${type}`);
                for (let i =0; i <  param[key].length; i++) {
                    let item = param[key][i];
                    if ( typeof item != 'number' ) {
                        throw new BizError(ERRORENUM.参数错误, `${key}应是number型数组其中下标${i}是${typeof item}`);
                    }
                }
            break;
            case 'Object':
                if ( type != 'object' ) throw new BizError(ERRORENUM.参数错误, `${key}应是Object型 而不是${type}`);
                if ( Array.isArray(param[key]) ) throw new BizError(ERRORENUM.参数错误, `${key}应是Object型 而不是array`);
                try {
                    JSON.stringify(param[key]);
                } catch(err) {
                    throw new BizError(ERRORENUM.参数错误, `${key}应是Object型 转换JSON时失败`); 
                }
                
            break;
        }
    }
    for (let key in param) {
        if (!conf[key]) throw new BizError(ERRORENUM.表单校验失败, `多余${key}字段`);
    }
    return param;
}

/**
 * 校验请求参数
 * 参数为非必填
 * @param conf 配置
 * @param param 待校验对象
 * @returns 原有表单数据
 */
export function eccReqNotMustHaveParam(conf, param) {
    for (let key in conf ) {
        if (!param[key]) continue;
        let type = typeof param[key];
        switch(conf[key]) {
            case 'Number':
                if ( type != 'number' ) throw new BizError(ERRORENUM.参数错误, `${key}应是number型 而不是${type}`);
            break;
            case 'String':
                if ( type != 'string' ) throw new BizError(ERRORENUM.参数错误, `${key}应是string型 而不是${type}`);
            break;
            case 'Boolean':
                if ( type != 'boolean' ) throw new BizError(ERRORENUM.参数错误, `${key}应是boolean型 而不是${type}`); 
            break;
            case '[Number]':
                if ( !Array.isArray(param[key]) ) throw new BizError(ERRORENUM.参数错误, `${key}应是数组型 而不是${type}`);
                for (let i =0; i <  param[key].length; i++) {
                    let item = param[key][i];
                    if ( typeof item != 'number' ) {
                        throw new BizError(ERRORENUM.参数错误, `${key}应是number型数组其中下标${i}是${typeof item}`);
                    }
                }
            break;
            case 'Object':
                if ( type != 'object' ) throw new BizError(ERRORENUM.参数错误, `${key}应是Object型 而不是${type}`);
                if ( Array.isArray(param[key]) ) throw new BizError(ERRORENUM.参数错误, `${key}应是Object型 而不是array`);
                try {
                    JSON.stringify(param[key]);
                } catch(err) {
                    throw new BizError(ERRORENUM.参数错误, `${key}应是Object型 转换JSON时失败`); 
                }
                
            break;
        }
    }
    for (let key in param) {
        if (!conf[key]) throw new BizError(ERRORENUM.表单校验失败, `多余${key}字段`);
    }
    return param;
}
